/**
 * QUnit Theme by NV v1.0.0
 *
 * https://github.com/Krinkle/qunit-theme-nv
 *
 * Copyright 2013 jQuery Foundation and other contributors
 * Released under the MIT license.
 * https://jquery.org/license/
 */

/** Font Family and Sizes */

#qunit-tests, #qunit-header, #qunit-banner, #qunit-testrunner-toolbar, #qunit-userAgent, #qunit-testresult {
	font-family: "Helvetica Neue", Helvetica, sans-serif;
}

#qunit-testrunner-toolbar, #qunit-userAgent, #qunit-testresult, #qunit-tests li { font-size: small; }
#qunit-tests { font-size: smaller; }


/** Resets */

#qunit-wrapper, #qunit-tests, #qunit-tests ol, #qunit-header, #qunit-banner, #qunit-userAgent, #qunit-testresult {
	margin: 0;
	padding: 0;
}


/** Header */

#qunit-header {
	font-family: "Helvetica Neue Light", "HelveticaNeue-Light", "Helvetica Neue", Helvetica, sans-serif;
	padding: 0.5em 0 0.5em 1.3em;

	color: #8699a4;
	background-color: #0d3349;

	font-size: 1.5em;
	line-height: 1em;
	font-weight: normal;
}

#qunit-header a {
	text-decoration: none;
	color: #c2ccd1;
}

#qunit-header a:hover,
#qunit-header a:focus {
	color: #fff;
}

#qunit-header label {
	display: inline-block;
}

#qunit-banner.qunit-pass {
	height: 3px;
}
#qunit-banner.qunit-fail {
	height: 5px;
}

#qunit-testrunner-toolbar {
	padding: .5em 0 0.5em 2em;
}

#qunit-testrunner-toolbar label {
	margin-right: 1em;
}

#qunit-modulefilter-container {
	float: right;
	margin-right: 1em;
}

#qunit-userAgent {
	padding: 0.5em 0 0.5em 2.5em;
	font-weight: normal;
	color: #666;
}


/** Tests: Pass/Fail */

#qunit-tests {
	list-style-type: none;
	background-color: #D2E0E6;
}

#qunit-tests li {
	padding: 0.4em 0.5em 0.4em 2.5em;
}

#qunit-tests li strong {
	font-weight: normal;
	cursor: pointer;
}

.qunit-assert-list {
	margin: 0.5em 0 1em;
	background-color: #fff;
}

.qunit-collapsed {
	display: none;
}

#qunit-tests table {
	border-collapse: collapse;
	margin-top: .2em;
}

#qunit-tests th {
	text-align: right;
	vertical-align: top;
	padding: 0 .5em 0 0;
}

#qunit-tests td {
	vertical-align: top;
}

#qunit-tests pre {
	margin: 0;
	white-space: pre-wrap;
	word-wrap: break-word;
}

#qunit-tests del {
	background-color: #e0f2be;
	color: #374e0c;
	text-decoration: none;
}

#qunit-tests ins {
	background-color: #ffcaca;
	color: #500;
	text-decoration: none;
}

/*** Test Counts */

#qunit-tests b.passed                       { color: #5E740B; }
#qunit-tests b.failed {
	color: #710909;
}
#qunit-tests li.fail .failed {
	color: #E48989;
}
#qunit-tests li.fail .passed {
	color: #E3C987;
}

#qunit-tests li li {
	margin-left: 2.5em;
	padding: 0.7em 0.5em 0.7em 0;
	background-color: #fff;
	border-bottom: none;
}

#qunit-tests b.counts {
	font-weight: normal;
}

/*** Passing Styles */

#qunit-tests li li.pass {
	color: #5E740B;
	background-color: #fff;
}

#qunit-tests .pass                          { color: #2f3424; background-color: #d9dec3; }
#qunit-tests .pass .module-name             { color: #636b51; }

#qunit-tests .pass .test-actual,
#qunit-tests .pass .test-expected           { color: #999999; }

#qunit-banner.qunit-pass                    { background-color: #C6E746; }

/*** Failing Styles */

#qunit-tests li li.fail {
	color: #710909;
	background-color: #fff;
}

#qunit-tests .fail                          { color: #fff; background-color: #962323; }
#qunit-tests .fail .module-name,
#qunit-tests .fail .counts                  { color: #DEC1C1; }

#qunit-tests .fail .test-actual             { color: #B72F2F; }
#qunit-tests .fail .test-expected           { color: green;   }

#qunit-banner.qunit-fail,
#qunit-testrunner-toolbar                   { color: #dec1c1; background-color: #962323; }


/** Footer */

#qunit-testresult {
	padding: 0.5em 0.5em 0.5em 2.5em;
	color: #333;
}
#qunit-testresult .module-name {
	font-weight: bold;
}

/** Fixture */

#qunit-fixture {
	position: absolute;
	top: -10000px;
	left: -10000px;
	width: 1000px;
	height: 1000px;
}
